const config = require("config");
const dayjs = require("dayjs");

class Order {
  static get STORAGE_KEY_PREFIX() {
    return "order_";
  }
  static get PURCHASE_STORAGE_TIME_MONTHS() {
    return 6;
  }
  // How many minutes is a user allowed to take for finishing the payment
  static get PURCHASE_TIME_LIMIT_MINUTES() {
    return 15;
  }

  static get PRICE_FOR_100() {
    return 2;
  }

  static get PRICE_FOR_250() {
    return 4;
  }

  /**
   * Data stored in Redis Database
   */
  #order_id;
  #amount;
  #unit_size;
  #price_per_unit;
  #encrypted_sales_receipts;
  #payment_completed;

  /**
   * Data populated by context
   */
  #create_mode;
  #redis_client;

  constructor(
    order_id,
    amount,
    unit_size,
    price_per_unit,
    encrypted_sales_receipts
  ) {
    this.#order_id = order_id;
    this.#amount = amount;
    this.#unit_size = unit_size;
    this.#price_per_unit = price_per_unit;
    this.#encrypted_sales_receipts = encrypted_sales_receipts;

    this.#payment_completed = false;
    this.#create_mode = true;

    let Redis = require("ioredis");
    this.#redis_client = new Redis({
      host: config.get("redis.host"),
    });
  }

  static async LOAD_ORDER_FROM_ID(order_id) {
    let Redis = require("ioredis");
    let redis_client = new Redis({
      host: config.get("redis.host"),
    });

    let redis_key = Order.STORAGE_KEY_PREFIX + order_id;
    let order_data = await redis_client.hgetall(redis_key);
    console.log(order_data);
  }

  async save() {
    let redis_key = Order.STORAGE_KEY_PREFIX + this.#order_id;
    let expiration = new dayjs();
    if (this.#create_mode) {
      expiration = expiration.add(Order.PURCHASE_TIME_LIMIT_MINUTES, "minute");
    } else {
      expiration = expiration.add(Order.PURCHASE_STORAGE_TIME_MONTHS, "month");
    }
    expiration = Math.round(expiration.diff() / 1000);

    let key_exists = await this.#redis_client.exists(redis_key);
    if (this.#create_mode && key_exists) {
      return Promise.reject("Cannot create a Order that already exists!");
    }

    let storage_promise = this.#redis_client
      .hmset(redis_key, {
        order_id: this.#order_id,
        amount: this.#amount,
        unit_size: this.#unit_size,
        price_per_unit: this.#price_per_unit,
        encrypted_sales_receipts: this.#encrypted_sales_receipts,
        payment_completed: this.#payment_completed,
      })
      .then((result) => {
        this.#create_mode = false;
      })
      .then((result) => {
        this.#redis_client.expire(redis_key, expiration);
      });

    await storage_promise;
  }
}

module.exports = Order;
