const BlindSignature = require("blind-signatures");
const Crypto = require("../../app/Crypto.js");
const Order = require("../../app/Order.js");
const config = require("config");

var express = require("express");
var router = express.Router();
const { query, body, validationResult } = require("express-validator");

/* GET home page. */
router.get(
  "/",
  query("amount")
    .isInt({ min: 0, max: 12 })
    .withMessage("Amount needs to be between 0 and 4."),
  async function (req, res, next) {
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
      return res.status(400).json({ errors: errors.array() });
    }

    let params = {
      amount: req.query.amount === 0 ? 1 : req.query.amount,
      unit_size: req.query.amount === 0 ? 100 : 250,
      price_per_unit:
        req.query.amount === 0 ? Order.PRICE_FOR_100 : Order.PRICE_FOR_250,
      order_id: await generate_unique_order_id(),
      payments: {
        paypal: {
          client_id: config.get("payments.paypal.testing.client_id"),
        },
      },
    };

    let crypto = new Crypto();
    let private_key = await crypto.private_key_get_current();
    params.crypto = {
      N: private_key.n,
      E: private_key.e,
    };

    // Generate hmac hash of the payment data so we are able to verify them when the client submits them again
    // Generate hmac hash of the payment data so we are able to verify them when the client submits them again
    params.integrity = crypto.createIntegrityHash(
      params.order_id,
      params.amount,
      params.unit_size,
      params.price_per_unit,
      params.crypto.N,
      params.crypto.E
    );

    res.render("checkout/checkout", params);
  }
);

router.use(
  "/payment/order",
  body("amount")
    .isInt({ min: 1, max: 4 })
    .withMessage("Invalid amount submitted")
    .toInt(),
  body("unit_size").isIn(["100", "250"]).toInt(),
  body("price_per_unit")
    .isCurrency({ symbol: "", allow_negatives: false, thousands_separator: "" })
    .withMessage("Invalid Price Value.")
    .toFloat(),
  body("order_id").isInt({ min: 0 }).withMessage("Invalid `order_id` value"),
  body("public_key_e")
    .isNumeric({ no_symbols: true })
    .withMessage("Invalid Public Key"),
  body("public_key_n")
    .isNumeric({ no_symbols: true })
    .withMessage("Invalid Public Key"),
  body("integrity")
    .isHash("sha256")
    .custom((value, { req }) => {
      let crypto = new Crypto();
      if (
        !crypto.validateIntegrityHash(
          value,
          req.body.order_id,
          req.body.amount,
          req.body.unit_size,
          req.body.price_per_unit,
          req.body.public_key_n,
          req.body.public_key_e
        )
      ) {
        return Promise.reject("Integrity is not matching");
      }
      return true;
    })
    .withMessage(
      "Value for `integrity` must be a SHA256 Hash and validate against the purchase data."
    ),
  body("encrypted_sales_receipts")
    .custom((value, { req }) => {
      if (!Array.isArray(value)) {
        return Promise.reject("Parameter is not an array");
      }
      for (let i = 0; i < value.length; i++) {
        if (!/^\d+$/.test(value[i])) {
          return Promise.reject(
            "Encrypted Sales Receipt contains invalid data"
          );
        }
      }
      if (value.length !== req.body.amount) {
        return Promise.reject("Two many receipts compared to the order.");
      }
      return true;
    })
    .withMessage("Invalid Sales Receipt"),
  (req, res, next) => {
    // This route gets called when the user initiates a payment: Validate the submitted data here
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
      return res.status(400).json({ errors: errors.array() });
    }

    // Order data is validated: Create and store the order in the redis database
    let order = new Order(
      req.body.order_id,
      req.body.amount,
      req.body.unit_size,
      req.body.price_per_unit,
      req.body.encrypted_sales_receipts
    );
    order
      .save()
      .then(() => {
        // Order created on our side. Continue the payment with the selected provider
        next("route");
      })
      .catch((reason) => {
        return res.status(400).json({
          errors: [
            {
              msg: reason,
            },
          ],
        });
      });
  }
);

var paypalRouter = require("./paypal.js");
const { PRICE_FOR_250 } = require("../../app/Order.js");
router.use("/payment/order/paypal", paypalRouter);

module.exports = router;

async function generate_unique_order_id() {
  // Generate Order ID => time in seconds since 1.1.1970 and and add a mutex to it to allow multiple order ids per second
  let Redis = require("ioredis");
  let redis_connection = new Redis({
    host: config.get("redis.host"),
  });
  let order_id = null;
  let order_base = Math.round(new Date().getTime() / 1000);
  let order_mutex = Math.floor(Math.random() * 10000);
  do {
    // make sure this order_id is not already registered
    let order_lock = await redis_connection.setnx(
      "" + order_base + order_mutex,
      true
    );
    if (order_lock === 1) {
      await redis_connection.expire(order_id, 5);
      order_id = "" + order_base + order_mutex;
      console.log(order_id);
    } else {
      console.log("Couldn't acquire lock");
      order_mutex++;
    }
  } while (order_id === null);
  return order_id;
}
